==> blcr-0.2.0.tar.gz: blcr-0.2.0/include/libcr.h <==
/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: libcr.h,v 1.26 2003/11/14 19:55:22 phargrov Exp $
 */

#ifndef _CRLIB_H
#define _CRLIB_H	1

#include <errno.h>
#include <features.h>
#include <cr.h>

// Maximum number of callbacks which can be registered
#define CR_MAX_CALLBACKS	32

// Type of callback functions
typedef int (*cr_callback_t)(void *);

// Integer type returned as an identifier from cr_init()
typedef int cr_client_id_t;

// Integer type returned as an identifier from cr_register_callback()
typedef int cr_callback_id_t;

// Type for signal-safe spinlocks
typedef unsigned long cr_spinlock_t;

// Type for cr_get_checkpoint_info()
struct cr_checkpoint_info {
	int		signal;		/* Signal that will be sent to the
					   process(es) when the checkpoint
					   has been taken, or zero if none
					   will be sent. */
};

// Values for return code from cr_status
enum {
	CR_STATE_IDLE,
	CR_STATE_PENDING,
	CR_STATE_ACTIVE
};

// Context flags for cr_{register,replace}_callback() and cr_replace_self().
#define CR_THREAD_CONTEXT	0x4000000
#define CR_SIGNAL_CONTEXT	0x2000000

// cr_client_id for use by callbacks
#define CR_ID_CALLBACK ((cr_client_id_t)(-1))

// Initializer for use with statically allocated spinlocks.
#define CR_SPINLOCK_INITIALIZER 0UL

__BEGIN_DECLS

// cr_init()
//
// Initialize (per-thread) state for use with libcr.
//
// This function must be called once from each thread before it makes
// any other calls to libcr functions.  This is not needed from
// threads which will never call libcr functions, nor is it needed
// inside of thread-context callback code (which runs in a thread
// created by libcr).
//
// On success, returns a non-negative identifier, of type
// cr_client_id_t, to be used in any subsequent calls to
// cr_{enter,leave}_cs().  This identifier has only thread-local scope
// and must not be used from another thread.
// Returns negative values to indicate failure.
//
// This function is not reentrant - do not call it from signal context.
//
extern cr_client_id_t
cr_init(void);

// cr_request()
//
// Request that a checkpoint be taken of the current process.  The checkpoint
// is not guaranteed to be complete by the time this function returns, but it
// is guaranteed that it will have reached either the CS_STATE_PENDING or
// CS_STATE_ACTIVE state.  So completion of the checkpoint can be polled for
// by calling cr_status() until it returns CS_STATE_IDLE.
//
// This function is not reentrant - do not call it from signal context.
//
extern void
cr_request(void);

// cr_register_callback()
//
// Register a checkpoint callback.
//
// Returns an identifier, of type cr_callback_id_t, to be used in any
// subsequent calls to cr_replace_callback().
//
// The 'func' argument is the callback to invoke at checkpoint time.
//
// The supplied 'arg' argument will be passed to your callback when
// invoked.
//
// The 'flags' argument is a bit-wise OR of flags.  The flags must
// include exactly one of following to specify in which context the
// callback may be run.
//   CR_THREAD_CONTEXT:
//	This type of registration is shared among all threads in the
//	program.  Thus, the ids returned from this call have global
//	scope.  The callback will be invoked in a dedicated checkpoint
//	thread.  The callback will NEVER be invoked within signal
//	context.  When multiple thread-context callbacks are
//	registered, the order in which they run with respect to each
//	other is not specified, and it is possible that they may run
//	concurrently.  It is guaranteed, however, that all
//	thread-context callbacks will reach their cr_checkpoint()
//	calls before ANY of signal-context callbacks run.
//   CR_SIGNAL_CONTEXT:
//	This type of registration is per-thread.  Thus, the ids
//	returned from this call have only per-thread scope.  The
//	callback will be invoked in the same thread which registered
//	it.  The callback MIGHT be invoked within signal-handler
//	context.  So, the callback must be capable of running to
//	completion in signal context.  When multiple signal-context
//	callbacks are registered in a given thread, they are called in
//	the reverse of the order they were registered, as with
//	atexit().  Signal-context callbacks are not run until all
//	thread-context callbacks (if any) have called cr_checkpoint().
//
// This function is not reentrant - do not call it from signal context.
//
extern cr_callback_id_t
cr_register_callback(cr_callback_t	func,
		     void*		arg,
		     int		flags);

// cr_replace_callback()
//
// Atomically replace one callback with another.
//
// Atomically (with respect to checkpoints and other cr_replace_*()
// calls) replaces the 'func', 'arg' and 'flags' associated with the
// given 'id'.  The new callback will be invoked in place of the
// function previously registered with this id, and in the same place
// in the ordering.
//
// There is currently no way to delete a callback id, but one can
// specify NULL for the 'func' argument to disable one.
//
// Returns 0 on success, nonzero on failure.
//
// This call can not change the context associated with the 'id'.
// Therefore, the 'flags' argument must include the same one of
// CR_SIGNAL_CONTEXT or CR_THREAD_CONTEXT which was originally used to
// register this 'id'.
//
// For ids registered with CR_SIGNAL_CONTEXT, it only makes sense to
// call this function from the thread which originally registered this
// id because signal-context ids have only thread-local scope.
//
// For ids registered with CR_THREAD_CONTEXT the ids have global
// scope.  Therefore this function maybe called from any thread which
// has previously called cr_init().
//
// This function is not reentrant - do not call it from signal context
// or from inside a callback.
//
extern int
cr_replace_callback(cr_callback_id_t	id,
		    cr_callback_t	func,
		    void*		arg,
		    int			flags);

// cr_enter_cs()
//
// Enter a critical section.
//
// Critical sections allow blocks of code to run atomically with
// respect to checkpoints and the associated callbacks.
//
// If invoked within any context other than a callback this routine
// ensures that checkpoint requests arriving while execution is in the
// critical section will be deferred.  This call may block if a
// checkpoint has been requested or is in progress.  In this context
// the 'id' argument must be the identifier returned from cr_init() in
// the calling thread.
//
// If invoked from within a callback this routine returns immediately.
// This allows the same code to be run from both inside and outside of
// callbacks without causing deadlock, presumably taking the 'id' as
// an argument.  In this context the 'id' must be the value
// CR_ID_CALLBACK.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_enter_cs(cr_client_id_t id);

// cr_leave_cs()
//
// Leave a critical section.
//
// Leave a critical section previously entered by calling
// cs_enter_cs().  If this call leaves the only critical section
// currently active, then any pending checkpoint will be taken before
// this call returns.
//
// The 'id' argument must be the same one used in a preceeding call to
// cr_enter_cs().  Calls to cr_enter_cs() and cr_leave_cs() must be
// balanced in the sense that the number of calls to cr_leave_cs()
// with a given id must never exceed the number of successful calls to
// cr_enter_cs() with the same id.  However, calls with distinct ids
// may be nested and/or overlapped arbitrarily.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_leave_cs(cr_client_id_t id);

// cr_status()
//
// Query the status of the checkpoint system.
//
// If this routine is called outside of a critical section one must
// be aware that the state could change between the time cr_status()
// returns and the time the return value is used.
//
// Returns an integer:
//  CR_STATE_IDLE	No checkpoint is pending or active.
//  CR_STATE_PENDING	A checkpoint request is pending
//  CR_STATE_ACTIVE	A checkpoint is active.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern int
cr_status(void);

// cr_checkpoint()
//
// Proceed to next step in the checkpoint.
//
// Called inside a callback to indicate that checkpoint may proceed.
//
// To abort the checkpoint and continue, pass CR_CHECKPOINT_TEMP_FAILURE for
// the 'flags' parameter.  To abort the checkpoint and kill all the processes
// involved in it, pass CR_CHECKPOINT_PERM_FAILURE.  Otherwise, pass
// CR_CHECKPOINT_READY (or 0).
//
// Returns:
//	Zero when continuing after taking a checkpoint. 
//	Positive when recovering from a checkpoint. 
//	Negative of error code if a checkpoint failed for some reason.
//	(man setjmp(3) for an analogue in POSIX.1).  If checkpoint was 
//	successfully aborted, the return value is -CR_ETEMPFAIL or
//	-CR_EPERMFAIL, depending on whether CR_CHECKPOINT_TEMP_FAILURE or
//	CR_CHECKPOINT_PERM_FAILURE was passed.
//
// Example callback:
//	int callback(void* arg)
//	{
//	    int rc;
//
//	    /* do CHECKPOINT work here */
//
//	    rc = cr_checkpoint(0);
//	    if (rc < 0) {
//		/* deal with FAILURE here */
//	    } else if (rc) {
//		/* do RESTART work here */
//	    } else {
//		/* do CONTINUE work here */
//	    }
//
//	    return 0;
//	}
// 
// Note that if a callback returns without invoking cr_checkpoint(),
// then it will be invoked immediately after the return.
//
// Callbacks are expected to return zero on success.
// A callback may return nonzero to indicate a failure to CONTIUNE or
// RESTART.  This is preferred over calling _exit(), but the exact
// behavior in this case is not yet defined.
//
// This routine may only be called from registered callbacks.
//
extern int
cr_checkpoint(int flags);

// cr_replace_self()
//
// Atomically replace the invoking callback with another.
//
// Atomically (with respect to checkpoints and other cr_replace_*()
// calls) replaces the 'func', 'arg' and 'flags' associated with the
// caller.
//
// There is currently no way to delete a callback id, but one can
// specify NULL for the 'func' argument to disable one.
//
// This call can not change the context associated with the callback.
// Therefore, the 'flags' argument must include the same one of
// CR_SIGNAL_CONTEXT or CR_THREAD_CONTEXT which was originally used to
// register the invoking callback.
//
// Returns 0 on success, nonzero on failure.
//
// This routine may only be called from registered callbacks.
//
extern int
cr_replace_self(cr_callback_t	func,
		void*		arg,
		int		flags);

// cr_get_checkpoint_info()
//
// Returns information about the active checkpoint request.
//
// The return value is a pointer to a structure in thread-local
// storage.  Therefore, reads of the structure are thread-safe
// within a running callback.  However, the values may change across
// calls to cr_checkpoint() if other callbacks run in the same thread.
// The contents of the structure are undefined when no checkpoint
// request is active.
//
// This routine may only be called from registered callbacks.
//
extern const struct cr_checkpoint_info *
cr_get_checkpoint_info(void);

// cr_spinlock_init()
//
// Initialize a spinlock (to the unlocked state).
//
// This must be done before the first use.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_init(cr_spinlock_t *);

// cr_spinlock_lock()
//
// Spin (busy wait) until the lock can be obtained.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_lock(cr_spinlock_t *);

// cr_spinlock_unlock()
//
// Unlock a lock held by this process/thread.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_unlock(cr_spinlock_t *);

// cr_spinlock_trylock()
//
// Try only once to obtain a lock.
//
// Returns non-zero if and only if the lock is obtained.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern int
cr_spinlock_trylock(cr_spinlock_t *);

// cr_strerror()
//
// Returns a string describing the error code passed in the 'ernum' argument.
// It behaves similarly to the standard C strerror() function, except that it
// can handle Berkeley CR-specific error codes in addition to the standard C
// ones.
extern char *
cr_strerror(int errnum);

__END_DECLS

#endif

==> blcr-0.5.0.tar.gz: blcr-0.5.0/include/libcr.h <==
/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: libcr.h,v 1.34 2007/01/05 23:47:36 phargrov Exp $
 */

#ifndef _CRLIB_H
#define _CRLIB_H	1

#include <errno.h>
#include <features.h>
#include <blcr_common.h>

// Maximum number of callbacks which can be registered
#define CR_MAX_CALLBACKS	32

// Type of callback functions
typedef int (*cr_callback_t)(void *);

// Integer type returned as an identifier from cr_init()
typedef int cr_client_id_t;

// Integer type returned as an identifier from cr_register_callback()
typedef int cr_callback_id_t;

// Type for signal-safe spinlocks
typedef unsigned int cr_spinlock_t;

// Type for cr_get_checkpoint_info()
struct cr_checkpoint_info {
	int		signal;		/* Signal that will be sent to the
					   process(es) when the checkpoint
					   has been taken, or zero if none
					   will be sent. */
	const char	*dest;		/* Path to the context file. */
};

// Type for cr_get_restart_info()
struct cr_restart_info {
	pid_t		requester;	/* pid of cr_restart */
	const char	*src;		/* Path to the context file. */
};

// Values for return code from cr_status
enum {
	CR_STATE_IDLE,
	CR_STATE_PENDING,
	CR_STATE_ACTIVE
};

// Context flags for cr_{register,replace}_callback() and cr_replace_self().
#define CR_THREAD_CONTEXT	0x4000000
#define CR_SIGNAL_CONTEXT	0x2000000

// cr_client_id for use by callbacks
#define CR_ID_CALLBACK ((cr_client_id_t)(-1))

// Initializer for use with statically allocated spinlocks.
#define CR_SPINLOCK_INITIALIZER 0U

__BEGIN_DECLS

// cr_init()
//
// Initialize (per-thread) state for use with libcr.
//
// This function must be called once from each thread before it makes
// any other calls to libcr functions.  This is not needed from
// threads which will never call libcr functions, nor is it needed
// inside of thread-context callback code (which runs in a thread
// created by libcr).
//
// On success, returns a non-negative identifier, of type
// cr_client_id_t, to be used in any subsequent calls to
// cr_{enter,leave}_cs().  This identifier has only thread-local scope
// and must not be used from another thread.
// Returns negative values to indicate failure.
//
// This function is not reentrant - do not call it from signal context.
//
extern cr_client_id_t
cr_init(void);

// cr_request()
//
// Request that a checkpoint be taken of the current process.  The checkpoint
// is not guaranteed to be complete by the time this function returns, but it
// is guaranteed that it will have reached either the CS_STATE_PENDING or
// CS_STATE_ACTIVE state.  So completion of the checkpoint can be polled for
// by calling cr_status() until it returns CS_STATE_IDLE.
//
// This function is not reentrant - do not call it from signal context.
//
extern void
cr_request(void);

// cr_request_file()
//
// Request that a checkpoint be taken of the current process.  The checkpoint
// is not guaranteed to be complete by the time this function returns, but it
// is guaranteed that it will have reached either the CS_STATE_PENDING or
// CS_STATE_ACTIVE state.  So completion of the checkpoint can be polled for
// by calling cr_status() until it returns CS_STATE_IDLE.
//
// The 'filename' argument is the name of the new context file.
//
// This function is not reentrant - do not call it from signal context.
//
extern void
cr_request_file(const char *filename);

// cr_request_fd()
//
// Request that a checkpoint be taken of the current process.  The checkpoint
// is not guaranteed to be complete by the time this function returns, but it
// is guaranteed that it will have reached either the CS_STATE_PENDING or
// CS_STATE_ACTIVE state.  So completion of the checkpoint can be polled for
// by calling cr_status() until it returns CS_STATE_IDLE.
//
// The context file will be written to the file descriptor given by the 'fd'
// argument.  The descriptor is not closed.
//
// This function is not reentrant - do not call it from signal context.
//
extern void
cr_request_fd(int fd);

// cr_register_callback()
//
// Register a checkpoint callback.
//
// Returns an identifier, of type cr_callback_id_t, to be used in any
// subsequent calls to cr_replace_callback().
//
// The 'func' argument is the callback to invoke at checkpoint time.
//
// The supplied 'arg' argument will be passed to your callback when
// invoked.
//
// The 'flags' argument is a bit-wise OR of flags.  The flags must
// include exactly one of following to specify in which context the
// callback may be run.
//   CR_THREAD_CONTEXT:
//	This type of registration is shared among all threads in the
//	program.  Thus, the ids returned from this call have global
//	scope.  The callback will be invoked in a dedicated checkpoint
//	thread.  The callback will NEVER be invoked within signal
//	context.  When multiple thread-context callbacks are
//	registered, the order in which they run with respect to each
//	other is not specified, and it is possible that they may run
//	concurrently.  It is guaranteed, however, that all
//	thread-context callbacks will reach their cr_checkpoint()
//	calls before ANY of signal-context callbacks run.
//   CR_SIGNAL_CONTEXT:
//	This type of registration is per-thread.  Thus, the ids
//	returned from this call have only per-thread scope.  The
//	callback will be invoked in the same thread which registered
//	it.  The callback MIGHT be invoked within signal-handler
//	context.  So, the callback must be capable of running to
//	completion in signal context.  When multiple signal-context
//	callbacks are registered in a given thread, they are called in
//	the reverse of the order they were registered, as with
//	atexit().  Signal-context callbacks are not run until all
//	thread-context callbacks (if any) have called cr_checkpoint().
//
// This function is not reentrant - do not call it from signal context.
//
extern cr_callback_id_t
cr_register_callback(cr_callback_t	func,
		     void*		arg,
		     int		flags);

// cr_replace_callback()
//
// Atomically replace one callback with another.
//
// Atomically (with respect to checkpoints and other cr_replace_*()
// calls) replaces the 'func', 'arg' and 'flags' associated with the
// given 'id'.  The new callback will be invoked in place of the
// function previously registered with this id, and in the same place
// in the ordering.
//
// There is currently no way to delete a callback id, but one can
// specify NULL for the 'func' argument to disable one.
//
// Returns 0 on success, nonzero on failure.
//
// This call can not change the context associated with the 'id'.
// Therefore, the 'flags' argument must include the same one of
// CR_SIGNAL_CONTEXT or CR_THREAD_CONTEXT which was originally used to
// register this 'id'.
//
// For ids registered with CR_SIGNAL_CONTEXT, it only makes sense to
// call this function from the thread which originally registered this
// id because signal-context ids have only thread-local scope.
//
// For ids registered with CR_THREAD_CONTEXT the ids have global
// scope.  Therefore this function maybe called from any thread which
// has previously called cr_init().
//
// This function is not reentrant - do not call it from signal context
// or from inside a callback.
//
extern int
cr_replace_callback(cr_callback_id_t	id,
		    cr_callback_t	func,
		    void*		arg,
		    int			flags);

// cr_enter_cs()
//
// Enter a critical section.
//
// Critical sections allow blocks of code to run atomically with
// respect to checkpoints and the associated callbacks.
//
// If invoked within any context other than a callback this routine
// ensures that checkpoint requests arriving while execution is in the
// critical section will be deferred.  This call may block if a
// checkpoint has been requested or is in progress.  In this context
// the 'id' argument must be the identifier returned from cr_init() in
// the calling thread.
//
// If invoked from within a callback this routine returns immediately.
// This allows the same code to be run from both inside and outside of
// callbacks without causing deadlock, presumably taking the 'id' as
// an argument.  In this context the 'id' must be the value
// CR_ID_CALLBACK.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_enter_cs(cr_client_id_t id);

// cr_leave_cs()
//
// Leave a critical section.
//
// Leave a critical section previously entered by calling
// cs_enter_cs().  If this call leaves the only critical section
// currently active, then any pending checkpoint will be taken before
// this call returns.
//
// The 'id' argument must be the same one used in a preceeding call to
// cr_enter_cs().  Calls to cr_enter_cs() and cr_leave_cs() must be
// balanced in the sense that the number of calls to cr_leave_cs()
// with a given id must never exceed the number of successful calls to
// cr_enter_cs() with the same id.  However, calls with distinct ids
// may be nested and/or overlapped arbitrarily.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_leave_cs(cr_client_id_t id);

// cr_status()
//
// Query the status of the checkpoint system.
//
// If this routine is called outside of a critical section one must
// be aware that the state could change between the time cr_status()
// returns and the time the return value is used.
//
// Returns an integer:
//  CR_STATE_IDLE	No checkpoint is pending or active.
//  CR_STATE_PENDING	A checkpoint request is pending
//  CR_STATE_ACTIVE	A checkpoint is active.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern int
cr_status(void);

// cr_checkpoint()
//
// Proceed to next step in the checkpoint.
//
// Called inside a callback to indicate that checkpoint may proceed.
//
// To abort the checkpoint and continue, pass CR_CHECKPOINT_TEMP_FAILURE for
// the 'flags' parameter.  To abort the checkpoint and kill all the processes
// involved in it, pass CR_CHECKPOINT_PERM_FAILURE.  Otherwise, pass
// CR_CHECKPOINT_READY (or 0).
//
// Returns:
//	Zero when continuing after taking a checkpoint. 
//	Positive when recovering from a checkpoint. 
//	Negative of error code if a checkpoint failed for some reason.
//	(man setjmp(3) for an analogue in POSIX.1).  If checkpoint was 
//	successfully aborted, the return value is -CR_ETEMPFAIL or
//	-CR_EPERMFAIL, depending on whether CR_CHECKPOINT_TEMP_FAILURE or
//	CR_CHECKPOINT_PERM_FAILURE was passed.
//
// Example callback:
//	int callback(void* arg)
//	{
//	    int rc;
//
//	    /* do CHECKPOINT work here */
//
//	    rc = cr_checkpoint(0);
//	    if (rc < 0) {
//		/* deal with FAILURE here */
//	    } else if (rc) {
//		/* do RESTART work here */
//	    } else {
//		/* do CONTINUE work here */
//	    }
//
//	    return 0;
//	}
// 
// Note that if a callback returns without invoking cr_checkpoint(),
// then it will be invoked immediately after the return.
//
// Callbacks are expected to return zero on success.
// A callback may return nonzero to indicate a failure to CONTIUNE or
// RESTART.  This is preferred over calling _exit(), but the exact
// behavior in this case is not yet defined.
//
// This routine may only be called from registered callbacks.
//
extern int
cr_checkpoint(int flags);

// cr_replace_self()
//
// Atomically replace the invoking callback with another.
//
// Atomically (with respect to checkpoints and other cr_replace_*()
// calls) replaces the 'func', 'arg' and 'flags' associated with the
// caller.
//
// There is currently no way to delete a callback id, but one can
// specify NULL for the 'func' argument to disable one.
//
// This call can not change the context associated with the callback.
// Therefore, the 'flags' argument must include the same one of
// CR_SIGNAL_CONTEXT or CR_THREAD_CONTEXT which was originally used to
// register the invoking callback.
//
// Returns 0 on success, nonzero on failure.
//
// This routine may only be called from registered callbacks.
//
extern int
cr_replace_self(cr_callback_t	func,
		void*		arg,
		int		flags);

// cr_get_checkpoint_info()
//
// Returns information about the active checkpoint request.
//
// The return value is a pointer to a structure in thread-local
// storage.  Therefore, reads of the structure are thread-safe
// within a running callback.  However, the values may change across
// calls to cr_checkpoint() if other callbacks run in the same thread.
// The contents of the structure are undefined when no checkpoint
// request is active.
//
// This routine may only be called from registered callbacks.
//
extern const struct cr_checkpoint_info *
cr_get_checkpoint_info(void);

// cr_get_restart_info()
//
// Returns information about the active restart request.
//
// The return value is a pointer to a structure in thread-local
// storage.  Therefore, reads of the structure are thread-safe
// within a running callback.  However, the values may change across
// calls to cr_checkpoint() if other callbacks run in the same thread.
// The contents of the structure are undefined when no restart
// request is active.
//
// This routine may only be called from registered callbacks.
//
extern const struct cr_restart_info *
cr_get_restart_info(void);

// cr_spinlock_init()
//
// Initialize a spinlock (to the unlocked state).
//
// This must be done before the first use.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_init(cr_spinlock_t *);

// cr_spinlock_lock()
//
// Spin (busy wait) until the lock can be obtained.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_lock(cr_spinlock_t *);

// cr_spinlock_unlock()
//
// Unlock a lock held by this process/thread.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern void
cr_spinlock_unlock(cr_spinlock_t *);

// cr_spinlock_trylock()
//
// Try only once to obtain a lock.
//
// Returns non-zero if and only if the lock is obtained.
//
// This routine is both thread-safe and reentrant (signal-safe).
//
extern int
cr_spinlock_trylock(cr_spinlock_t *);

// cr_strerror()
//
// Returns a string describing the error code passed in the 'ernum' argument.
// It behaves similarly to the standard C strerror() function, except that it
// can handle Berkeley CR-specific error codes in addition to the standard C
// ones.
extern const char *
cr_strerror(int errnum);

__END_DECLS

#endif
